// Cutting Marks

// JavaScript Script for Adobe Illustrator CS3
// Tested with Adobe Illustrator CS3 13.0.3, Windows XP SP3 (Japanese version).
// This script provided "as is" without warranty of any kind.
// Free to use and distribute.

// Copyright(c) 2010 Hiroyuki Sato
// http://lines-about-to-be-generated.blogspot.com/


var g_help = "\r\rUsage: select pathes and a group as a cutting mark sample,\r"
+ "then run this script.";

var g_len0, g_len1, g_mark, g_marks;

main();
function main(){
  
  // gets the selection
  var sel = app.activeDocument.selection;
  if(sel.length < 2){
    alert("Error: There must be 2 or more items in the selection." + g_help);
    return;
  }

  // finds a group as trim-mark in the selection
  var i, j;
  var gi = null;
  
  for(i=0; i < sel.length; i++){
    if(sel[i].typename == "GroupItem"){
      gi = sel[i];
      break;
    }
  }

  if(!gi){
    alert("Error: There must be a group as cutting mark in the selection." + g_help);
    return;
  }
    
  if(gi.pathItems.length < 1){
    alert("Error: The 1st group in the selection must contain a path." + g_help);
    return;
  }
  
  g_mark = gi.pathItems[0];
  if(g_mark.pathPoints.length != 2){
    alert("Error: The 1st path in the 1st group must have 2 pathPoints." + g_help);
    return;
  }

  var center = findCenter(gi);
    
  g_len0 = dist( g_mark.pathPoints[0].anchor,
                 center );
  g_len1 = dist( g_mark.pathPoints[1].anchor,
                 center );


  // draws trim-marks
  var pp;

  g_marks = g_mark.layer.groupItems.add();
  g_marks.name = "cutting marks";
  
  for(i=0; i<sel.length; i++){
    if(sel[i].typename != "PathItem") continue;
    
    pp = sel[i].pathPoints;
  
    for(var j=0; j < pp.length; j++){
      createCuttingMark(pp, j, j+1);
      createCuttingMark(pp, j, j-1);
    }
  }

  if(g_marks.pathItems.length < 1){
    g_marks.remove();
  }
}

// ---------------------
function createCuttingMark(pp, idx0, idx1){
  idx1 = parseIdx( pp, idx1 );
  
  if( idx1 >= 0 ){
    var t = getRad(pp[idx1].anchor, pp[idx0].anchor);
    
    var mark = g_mark.duplicate();
    mark.move(g_marks, ElementPlacement.PLACEATBEGINNING);
    
    var p = getPoint(pp[idx0].anchor, t, g_len0);
    setCorner( mark.pathPoints[0], p);
  
    p = getPoint(pp[idx0].anchor, t, g_len1);
    setCorner( mark.pathPoints[1], p);
  }
}

// ---------------------
function findCenter(pi){
  var gb = pi.geometricBounds; // left, top, right, bottom
  return [(gb[0]+gb[2])/2, (gb[1]+gb[3])/2];
}

// ---------------------
function getRad(p1, p2) {
  return Math.atan2(p2[1] - p1[1], p2[0] - p1[0]);
}

// ---------------------
function getPoint(anc, t, len){
  return [ Math.cos(t) * len + anc[0],
           Math.sin(t) * len + anc[1]];
}

// ---------------------
function setCorner(p, pnt){
  with(p){
    anchor = pnt;
    rightDirection = pnt;
    leftDirection = pnt;
  }
}

// ---------------------
function parseIdx(p, n){ // PathPoints, number for index
  var len = p.length;
  if(p.parent.closed){
    return n>=0 ? n % len : len - Math.abs(n % len);
  } else {
    return (n<0 || n>len-1) ? -1 : n;
  }
}

// ---------------------
function dist(p1, p2) {
  return Math.sqrt(Math.pow(p1[0] - p2[0],2) + Math.pow(p1[1] - p2[1],2));
}
